#include "stdafx.h"
#include "listPrint.h"

#include "helper.h"

#include "listPrintDoc.h"
#include "listPrintView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CListPrintView

IMPLEMENT_DYNCREATE(CListPrintView, CListView)

BEGIN_MESSAGE_MAP(CListPrintView, CListView)
	//{{AFX_MSG_MAP(CListPrintView)
	ON_COMMAND(ID_FILL, OnFill)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_COMMAND(ID_BUTTON_SETUP, OnButtonSetup)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_SETUP, OnUpdateButtonSetup)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_PRINT_DIRECT, OnFilePrintDirect)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_DIRECT, OnUpdateFilePrintDirect)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CListPrintView Konstruktion/Destruktion

CListPrintView::CListPrintView()
{
	m_nCols = 11;
	m_nRows	= 80;
	
	m_bFilled		= FALSE;
	m_bUseSetup		= TRUE;
}

CListPrintView::~CListPrintView()
{
}

void CListPrintView::OnInitialUpdate()
{
	CListCtrl& listctrl = GetListCtrl();

	OnViewStyle(LVS_REPORT);
	CListView::OnInitialUpdate();

	AfxMessageBox(IDS_START);
}

CListPrintDoc* CListPrintView::GetDocument() // Die endgltige (nicht zur Fehlersuche kompilierte) Version ist Inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CListPrintDoc)));
	return (CListPrintDoc*)m_pDocument;
}

void CListPrintView::OnViewStyle(UINT nCommandID)
{
	switch (nCommandID)
	{
		case ID_VIEW_LINEUP:
			{
				CListCtrl& refListCtrl = GetListCtrl();
				refListCtrl.Arrange(LVA_SNAPTOGRID);
			}
			break;

		default:	// LVS_REPORT, LVS_SMALLICON, LVS_ICON, LVS_LIST
			ModifyStyle(LVS_TYPEMASK, nCommandID);
			break;
	}
}

void CListPrintView::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if( !m_bFilled )
		return;
	
	CListCtrl		&listctrl = GetListCtrl();

	NM_LISTVIEW		*pNMListView = (NM_LISTVIEW*)pNMHDR;
	int				iCol = pNMListView->iSubItem,
					amount = m_ColSortOrder.GetSize();
	BOOL			bSortOrder = 1;

	if( iCol>=amount )
		m_ColSortOrder.SetSize(iCol+1);

	bSortOrder = m_ColSortOrder[iCol];
	bSortOrder++;
	bSortOrder%=2;
	m_ColSortOrder[iCol] = bSortOrder;
	
	::SortListCtrl(&listctrl, iCol, bSortOrder);
	
	*pResult = 0;

	Invalidate();
	UpdateWindow();
}

void CListPrintView::OnFill() 
{
	CWaitCursor wait;
	CListCtrl	&refCtrl = GetListCtrl();
	int			i,
				j,
				nCol=0,
				colId,
				iActualItem;
	CString		str;

	if( m_bFilled )
	{
		AfxMessageBox(IDS_STUPID);
		return;
	}

	m_bFilled	= TRUE;

	// speed up !
	SetRedraw(FALSE);

	//
	// create Columns ...
	//
	for( i=0 ; i<m_nCols ; i++ )
	{
		// create columns
		str.Format(IDS_COLUMN, i+1);
		colId = refCtrl.InsertColumn(nCol, str, LVCFMT_LEFT, 100);
		nCol++;

	}
	m_ColSortOrder.SetSize(nCol);

	// ... and fill in text
	for( j=0; j<m_nRows ; j++ )
	{
		str.Format(IDS_CELL, i+1, j+1);
		iActualItem = refCtrl.InsertItem(j, str);
		
		for( i=0 ; i<m_nCols ; i++ )
		{
			str.Format(IDS_CELL, i+1, j+1);
			refCtrl.SetItemText(iActualItem, i, str);
		}
	}


	DWORD dwStyle = refCtrl.SendMessage(LVM_GETEXTENDEDLISTVIEWSTYLE);
	dwStyle |= LVS_EX_HEADERDRAGDROP| LVS_EX_GRIDLINES;
	dwStyle |= LVS_EX_FULLROWSELECT;

	refCtrl.SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE, 0, (LPARAM)dwStyle);

	SetRedraw(TRUE);
}

void CListPrintView::OnFilePrintDirect() 
{
	CWaitCursor		wait;
	CListCtrl		&listctrl = GetListCtrl();

	CListCtrlPrintJob	job(&listctrl, !m_bUseSetup);
	job.Print();
}

void CListPrintView::OnUpdateFilePrintDirect(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bFilled);
}


void CListPrintView::OnButtonSetup() 
{
	m_bUseSetup ++ ;
	m_bUseSetup %= 2;
	
}

void CListPrintView::OnUpdateButtonSetup(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bUseSetup);
}
